local VERSION = tonumber(('$Revision: 32463 $'):match('%d+'))

local PitBull = PitBull
local PitBull_Aura = PitBull:NewModule('Aura', 'AceEvent-2.0', 'AceConsole-2.0')
local self = PitBull_Aura
if PitBull.revision < VERSION then
	PitBull.version = 'r' .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ('$Date: 2007-04-12 11:26:17 -0400 (Thu, 12 Apr 2007) $'):match('%d%d%d%d%-%d%d%-%d%d')
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local GameTooltip = GameTooltip
local UnitBuff = UnitBuff
local UnitDebuff = UnitDebuff
local math_floor = math.floor
local math_min = math.min
local playerClass

local canDispel = {
	["PRIEST"] = {
		["Magic"] = true,
		["Disease"] = true,
	},
	["SHAMAN"] = {
		["Poison"] = true,
		["Disease"] = true,
	},
	["PALADIN"] = {
		["Magic"] = true,
		["Poison"] = true,
		["Disease"] = true,
	},
	["MAGE"] = {
		["Curse"] = true,
	},
	["DRUID"] = {
		["Curse"] = true,
		["Poison"] = true,
	},
}

PitBull_Aura:RegisterPitBullChildFrames('auraFrame', 'auraFrame2', 'debuffhighlight')

local unitsShown = {}

function PitBull_Aura:OnInitialize()
	self.db = PitBull:AcquireDBNamespace('Aura')
	PitBull:RegisterDefaults('Aura', 'profile', {
		['**'] = {
			hidden = false,
			hiddenBuffs = false,
			hiddenDebuffs = false,
			highlight = { Buffs = true, Debuffs = true},
			highlightType = true,
			split = false,
			snap = false,
			filter = true,
			maxbuffs = 32,
			maxdebuffs = 40,
			buffsize = 16,
			debuffsize = 16,
			flip = false,
			grow = -1,
			buffdirection = "Right then Down",
			debuffdirection = "Right then Down",
			frameHighlight = "Cureable by me",
		},
		target = {
			filter = false,
			frameHighlight = "Never",
		},
		colors = {
			buffs = {0, 1, 0, 1},
			debuffs = {
				Poison = {0, 1, 0, 1},
				Magic = {0, 0, 1, 1},
				Disease = {.55, .15, 0, 1},
				Curse = {5, 0, 5, 1},
				['nil'] = {1, 0, 0, 1},
			}
		}
	})
end

function PitBull_Aura:OnEnable(first)
	self:RegisterBucketEvent('UNIT_AURA', 0.05)
	playerClass = select(2, UnitClass("player") )
end

function PitBull_Aura:UNIT_AURA(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateAuras(unit, frame)
		end
	end
end

local function buffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT')
	GameTooltip:SetUnitBuff(this.unit, this.id, this.raidFilter)
end

local function debuffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT')
	GameTooltip:SetUnitDebuff(this.unit, this.id, this.raidFilter)
end

local function buffFrame_OnLeave(this)
	GameTooltip:Hide()
end
local debuffFrame_OnLeave = buffFrame_OnLeave

local function UpdateBuffs(unit, frame)
	local split = PitBull_Aura.db.profile[frame.group].split
	local snap = PitBull_Aura.db.profile[frame.group].snap
	local buffs = frame.auraFrame.buffs or new()
	local raidFilter = PitBull_Aura.db.profile[frame.group].filter and 1 or 0
	local size = PitBull_Aura.db.profile[frame.group].buffsize
	local maxbuffs = PitBull_Aura.db.profile[frame.group].maxbuffs
	local max_buffs_perrow = math_floor(frame.auraFrame:GetWidth() / size)
	local scale_adjust = frame.auraFrame:GetWidth() / (size * max_buffs_perrow)
	local font, fontsize = PitBull:GetFont()
	
	if snap then
		size = size * scale_adjust
	end
	
	for i = 1, 40 do
		local texture, applications
		
		_, _, texture, applications = UnitBuff(unit, i, raidFilter)
		if not texture then --and not PitBull.configMode then
			for j = #buffs, i, -1 do
				local v = buffs[j]
				buffs[j] = nil
				v.text = delFrame(v.text)
				if v.border then
					v.border = delFrame(v.border)
				end
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v:SetScript('OnEnter', nil)
				v:SetScript('OnLeave', nil)
				delFrame(v)
			end
			break
		end
		
		local buffFrame = buffs[i] or newFrame('Button', frame.auraFrame)
		buffFrame:ClearAllPoints()
		buffFrame:SetWidth(size)
		buffFrame:SetHeight(size)
		local text = buffFrame.text or newFrame('FontString', buffFrame, 'OVERLAY')
		buffFrame.text = text
		text:SetFont( font, fontsize )
		text:SetShadowColor( 0, 0, 0, 1)
		text:SetShadowOffset( 0.8, -0.8 )
		if PitBull.configMode and not texture then
			text:SetText('5')
			text:SetPoint('BOTTOMRIGHT', buffFrame, 'BOTTOMRIGHT', -1, 1)
			local icon = buffFrame.icon or newFrame('Texture', buffFrame, 'BACKGROUND')
			buffFrame.icon = icon
			icon:SetTexture("Interface\\Icons\\Ability_Shaman_WaterShield")
			icon:SetAllPoints(buffFrame)
			if PitBull_Aura.db.profile[frame.group].highlight.Buffs then
				local border = buffFrame.border or newFrame('Texture', buffFrame, 'BORDER')
				buffFrame.border = border
				border:ClearAllPoints()
				border:SetAllPoints(buffFrame)
				border:SetTexture('Interface\\AddOns\\PitBull\\Aura\\debuffHighlight')
				border:SetVertexColor(unpack(self.db.profile.colors.buffs))
			end
			buffs[i] = buffFrame
		else
			if applications <= 1 then
				text:SetText('')
			else
				text:SetText(applications)
			end
			text:SetPoint('BOTTOMRIGHT', buffFrame, 'BOTTOMRIGHT', -1, 1)
			local icon = buffFrame.icon or newFrame('Texture', buffFrame, 'BACKGROUND')
			buffFrame.icon = icon
			icon:SetTexture(texture)
			icon:SetAllPoints(buffFrame)
			if PitBull_Aura.db.profile[frame.group].highlight.Buffs then
				local border = buffFrame.border or newFrame('Texture', buffFrame, 'BORDER')
				buffFrame.border = border
				border:ClearAllPoints()
				border:SetAllPoints(buffFrame)
				border:SetTexture('Interface\\AddOns\\PitBull\\Aura\\debuffHighlight')
				border:SetVertexColor(unpack(self.db.profile.colors.buffs))
			end
			buffFrame:SetScript('OnEnter', buffFrame_OnEnter)
			buffFrame:SetScript('OnLeave', buffFrame_OnLeave)
			buffFrame.id = i
			buffFrame.unit = unit
			buffFrame.raidFilter = raidFilter
			buffs[i] = buffFrame
		end
	end
	local num_buffs = #buffs
	
	local num_shown_buffs = math_min(num_buffs, maxbuffs)
	
	local row = 1
	local lastrow = 1
	
	local direction = PitBull_Aura.db.profile[frame.group].buffdirection
	for i = 1, num_shown_buffs do
		local buff = buffs[i]
		buff:Show()
		buff:ClearAllPoints()
		if split then
			if direction == "Up then Left" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', -1 * (row - 1) * size, ((i-1) % max_buffs_perrow) * size)
			elseif direction == "Up then Right" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', (row - 1) * size, ((i-1) % max_buffs_perrow) * size)
			elseif direction == "Down then Left" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', -1 * (row - 1) * size, -1 * ((i-1) % max_buffs_perrow) * size)
			elseif direction == "Down then Right" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', (row - 1) * size, -1 * ((i-1) % max_buffs_perrow) * size)
			elseif direction == "Left then Up" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', -1 * ((i-1) % max_buffs_perrow) * size, (row - 1) * size)
			elseif direction == "Left then Down" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', -1 * ((i-1) % max_buffs_perrow) * size, -1 * (row - 1) * size)
			elseif direction == "Right then Up" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', ((i-1) % max_buffs_perrow) * size, (row - 1) * size)
			elseif direction == "Right then Down" then
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', ((i-1) % max_buffs_perrow) * size, -1 * (row - 1) * size)
			end
		else
			if PitBull_Aura.db.profile[frame.group].flip then
				buff:SetPoint('TOPRIGHT', frame.auraFrame, 'TOPRIGHT', ((i-1) % max_buffs_perrow) * -1 * size, (row - 1) * self.db.profile[frame.group].grow * size)
			else
				buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', ((i-1) % max_buffs_perrow) * size, (row - 1) * self.db.profile[frame.group].grow * size)
			end
		end
		lastrow = row
		if i % max_buffs_perrow == 0 then
			row = row + 1
		end
	end
	for i = num_shown_buffs + 1, num_buffs do
		if i > 0 then
			local buff = buffs[i]
			buff:Hide()
			buff:ClearAllPoints()
		end
	end
	
	if num_buffs == 0 then
		buffs = del(buffs)
	end
	
	frame.auraFrame:SetHeight(lastrow * size)
	frame.auraFrame.buffs = buffs
	frame.auraFrame.num_shown_buffs = num_shown_buffs
end

local function UpdateDebuffs(unit, frame, buffs)
	local split = PitBull_Aura.db.profile[frame.group].split
	local snap = PitBull_Aura.db.profile[frame.group].snap	
	local raidFilter = PitBull_Aura.db.profile[frame.group].filter and 1 or 0
	local size = PitBull_Aura.db.profile[frame.group].debuffsize
	local auraFrame_width = frame.auraFrame:GetWidth()
	local max_debuffs_perrow = split and math_floor(frame.auraFrame2:GetWidth() / size) or math_floor(auraFrame_width / size)	
	local scale_adjust = auraFrame_width / (size * max_debuffs_perrow)
	local font, fontsize = PitBull:GetFont()
	
	if snap then
		size = size * scale_adjust
	end
	
	local debuffs 
	if split then
		debuffs = frame.auraFrame2.debuffs or new()
	else
		debuffs = frame.auraFrame.debuffs or new()
	end
	
	local maxdebuffs = PitBull_Aura.db.profile[frame.group].maxdebuffs
	
	local texture, applications, dispelType

	for i = 1, 40 do
		_, _, texture, applications, dispelType = UnitDebuff(unit, i, raidFilter)

		if not texture then -- and not PitBull.configMode then
			for j = #debuffs, i, -1 do
				local v = debuffs[j]
				debuffs[j] = nil
				v.text = delFrame(v.text)
				if v.border then
					v.border = delFrame(v.border)
				end
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v.raidFilter = nil
				v:SetScript('OnEnter', nil)
				v:SetScript('OnLeave', nil)
				delFrame(v)
			end
			break
		end
		
		local debuffFrame = debuffs[i] or newFrame('Button', split and frame.auraFrame2 or frame.auraFrame)
		debuffFrame:ClearAllPoints()
		debuffFrame:SetWidth(size)
		debuffFrame:SetHeight(size)
		local text = debuffFrame.text or newFrame('FontString', debuffFrame, 'OVERLAY')
		debuffFrame.text = text
		text:SetFont( font, fontsize )
		text:SetShadowColor( 0, 0, 0, 1)
		text:SetShadowOffset( 0.8, -0.8 )
		if PitBull.configMode and not texture then
			text:SetText('5')
			text:SetPoint('BOTTOMRIGHT', debuffFrame, 'BOTTOMRIGHT', -1, 1)
			local icon = debuffFrame.icon or newFrame('Texture', debuffFrame, 'BACKGROUND')
			debuffFrame.icon = icon
			icon:SetTexture("Interface\\Icons\\Ability_Creature_Cursed_03")
			icon:SetAllPoints(debuffFrame)
			if PitBull_Aura.db.profile[frame.group].highlight.Debuffs then
				local border = debuffFrame.border or newFrame('Texture', debuffFrame, 'BORDER')
				debuffFrame.border = border
				border:ClearAllPoints()
				border:SetAllPoints(debuffFrame)
				border:SetTexture('Interface\\AddOns\\PitBull\\Aura\\debuffHighlight')
				border:SetVertexColor(unpack(self.db.profile.colors.debuffs['nil']))
			end
			debuffs[i] = debuffFrame
		else
			if applications <= 1 then
				text:SetText('')
			else
				text:SetText(applications)
			end
			text:SetPoint('BOTTOMRIGHT', debuffFrame, 'BOTTOMRIGHT', -1, 1)
			local icon = debuffFrame.icon or newFrame('Texture', debuffFrame, 'BACKGROUND')
			debuffFrame.icon = icon
			icon:SetTexture(texture)
			icon:SetAllPoints(debuffFrame)
			if PitBull_Aura.db.profile[frame.group].highlight.Debuffs then
				local border = debuffFrame.border or newFrame('Texture', debuffFrame, 'BORDER')
				debuffFrame.border = border
				border:ClearAllPoints()
				border:SetAllPoints(debuffFrame)
				border:SetTexture('Interface\\AddOns\\PitBull\\Aura\\debuffHighlight')
				if PitBull_Aura.db.profile[frame.group].highlightType then
					border:SetVertexColor(unpack(self.db.profile.colors.debuffs[dispelType or 'nil']))
				else
					border:SetVertexColor(unpack(self.db.profile.colors.debuffs['nil']))
				end
			end
			debuffFrame:SetScript('OnEnter', debuffFrame_OnEnter)
			debuffFrame:SetScript('OnLeave', debuffFrame_OnLeave)
			debuffFrame.id = i
			debuffFrame.unit = unit
			debuffFrame.raidFilter = raidFilter
			debuffs[i] = debuffFrame
		end
	end

	local num_debuffs = #debuffs
		
	local num_shown_debuffs = math_min(num_debuffs, maxdebuffs)
	
	if split then
		-- note that this is the top row that we'll be using for the first few debuffs.  split mode they descend, nonsplit they ascend
		local direction = PitBull_Aura.db.profile[frame.group].debuffdirection
		local row = 1
		for i = 1, num_shown_debuffs do
			local debuff = debuffs[i]
			debuff:Show()
			debuff:SetBackdropBorderColor(1,0,0)
			debuff:ClearAllPoints()
			if direction == "Up then Left" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', -1 * (row - 1) * size, ((i-1) % max_debuffs_perrow) * size)
			elseif direction == "Up then Right" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', (row - 1) * size, ((i-1) % max_debuffs_perrow) * size)
			elseif direction == "Down then Left" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', -1 * (row - 1) * size, -1 * ((i-1) % max_debuffs_perrow) * size)
			elseif direction == "Down then Right" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', (row - 1) * size, -1 * ((i-1) % max_debuffs_perrow) * size)
			elseif direction == "Left then Up" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', -1 * ((i-1) % max_debuffs_perrow) * size, (row - 1) * size)
			elseif direction == "Left then Down" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', -1 * ((i-1) % max_debuffs_perrow) * size, -1 * (row - 1) * size)
			elseif direction == "Right then Up" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', ((i-1) % max_debuffs_perrow) * size, (row - 1) * size)
			elseif direction == "Right then Down" then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', ((i-1) % max_debuffs_perrow) * size, -1 * (row - 1) * size)
			end
			if i % max_debuffs_perrow == 0 then
				row = row + 1
			end
		end
	else
		local buffsize = PitBull_Aura.db.profile[frame.group].buffsize
		
		local max_buffs_perrow = math_floor(auraFrame_width / buffsize)
		local num_buffs = frame.auraFrame.num_shown_buffs or 0
		
		local num_buffs_bottomrow = num_buffs % max_buffs_perrow
		local num_debuffs_toprow = num_shown_debuffs % max_debuffs_perrow
		
		local colliding = (num_debuffs_toprow * size + num_buffs_bottomrow * buffsize) > auraFrame_width
		
		-- start at the bottom full row of buffs
		local offset = math_floor(num_buffs / max_buffs_perrow) * buffsize +
		-- if colliding, push it down the sum of the values
		(colliding and (buffsize + size) or 0) +
		-- if sharing a row, push it down the larger value.
		((not colliding and num_debuffs_toprow > 0 and num_buffs_bottomrow > 0) and (buffsize > size and buffsize or size) or 0) +
		-- if one row's full and the other isnt, push it the pertaining value.
		((num_debuffs_toprow == 0 and num_buffs_bottomrow > 0) and buffsize or 0) +
		((num_debuffs_toprow > 0 and num_buffs_bottomrow == 0) and size or 0)
		
		local flip = PitBull_Aura.db.profile[frame.group].flip
		
		local grow = self.db.profile[frame.group].grow
		offset = grow == 1 and offset - buffsize + size or grow * offset
		
		-- note that this will be the bottom row that we draw debuffs on.
		local row = math_floor(num_shown_debuffs / max_debuffs_perrow)
		for i = 1, num_shown_debuffs do
			local debuff = debuffs[i]
			debuff:Show()
			debuff:SetBackdropBorderColor(1,0,0)
			debuff:ClearAllPoints()
			if (flip and buffs) or (not flip and not buffs) then
				debuff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', ((i-1) % max_debuffs_perrow) * size, (row - 1) * grow * size + offset)
			else
				debuff:SetPoint('TOPRIGHT', frame.auraFrame, 'TOPRIGHT', ((i-1) % max_debuffs_perrow) * -1 * size, (row - 1) * grow * size + offset)
			end
			if i % max_debuffs_perrow == 0 then
				row = row - 1
			end
		end
	end
	for i = num_shown_debuffs + 1, num_debuffs do
		if i > 0 then
			local debuff = debuffs[i]
			debuff:Hide()
			debuff:ClearAllPoints()
		end
	end
	
	if num_debuffs == 0 then
		debuffs = del(debuffs)
	end
	
	if split then
		frame.auraFrame2.debuffs = debuffs
	else
		frame.auraFrame.debuffs = debuffs
	end
end

local function UpdateFrameHighlight(unit, frame )
	local frameHighlight = PitBull_Aura.db.profile[frame.group].frameHighlight
	
	local htexture, hdispel, hashighlight, checkagain

	for i = 1, 40 do
		if not hashighlight or hashighlight == 'nil' or checkagain then -- if hashighlight is 'nil' we keep trying there might be a curable debuff we prioritize those for highlighting
			_, _, htexture, _, hdispel = UnitDebuff(unit, i, 0) -- ignore the raidfilter for frame highlighting
			if htexture then
				if not hdispel then -- no dispel type set
					if frameHighlight == "All Debuffs" then -- only set highlight to 'nil'  if we want to show all debuffs
						hashighlight = 'nil' -- we specifically continue here to find an optional cureable or cureable by me debuff which has priority for highlighting
					end 
				elseif frameHighlight == "Cureable" or frameHighlight == "All Debuffs" then
					if canDispel[playerClass] and canDispel[playerClass][hdispel] then
						hashighlight = hdispel
						break -- break here, we've got a cureable by me debuff...
					else
						hashighlight = hdispel -- continue here to try and find a cureable by me debuff.
					end
				elseif frameHighlight == "Cureable by me" and canDispel[playerClass] and canDispel[playerClass][hdispel] then
					hashighlight = hdispel
					break -- break here, we've got a cureable by me debuff...
				end
			else
				break
			end
		end
	end

	if hashighlight then
		frame.debuffhighlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
		frame.debuffhighlight:SetBlendMode("ADD")
		frame.debuffhighlight:SetAlpha(0.5)	
		frame.debuffhighlight:SetVertexColor(unpack(self.db.profile.colors.debuffs[hashighlight]))
		frame.debuffhighlight:Show()
	else
		frame.debuffhighlight:Hide()
	end	
	
end

function PitBull_Aura:UpdateAuras(unit, frame)
	if self.db.profile[frame.group].frameHighlight == "Never" then
		frame.debuffhighlight:Hide()
	else
		UpdateFrameHighlight(unit, frame)
	end
	if not unitsShown[unit] or self.db.profile[frame.group].hidden then
		return
	end
	local buffs
	if not self.db.profile[frame.group].hiddenBuffs then
		buffs = true
		UpdateBuffs(unit, frame)
	end
	if not self.db.profile[frame.group].hiddenDebuffs then
		UpdateDebuffs(unit, frame, buffs)
	end
end
PitBull_Aura.OnUpdateFrame = PitBull_Aura.UpdateAuras

function PitBull_Aura:OnPopulateUnitFrame(unit, frame)
	frame.auraFrame = newFrame('Frame')
	if self.db.profile.split then
		frame.auraFrame2 = newFrame('Frame')
	end
	local highlight = newFrame("Texture", frame.overlay, "OVERLAY")
	frame.debuffhighlight = highlight
	highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	highlight:SetBlendMode("ADD")
	highlight:SetAlpha(0.5)
	highlight:Hide()
	highlight:SetAllPoints(frame)
end

function PitBull_Aura:OnUpdateFont(font, fontsize)
	for unit,frame in PitBull:IterateUnitFrames() do
		self:UpdateAuras(unit, frame)
	end
end

function PitBull_Aura:ClearAuras(unit, frame)
	local buffs = frame.auraFrame.buffs
	if buffs then
		for i = #buffs, 1, -1 do
			local v = buffs[i]
			buffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			if v.border then
				v.border = delFrame(v.border)
			end
			v.unit = nil
			v.id = nil
			v.raidFilter = nil
			v:SetScript('OnEnter', nil)
			v:SetScript('OnLeave', nil)
			delFrame(v)
		end
		frame.auraFrame.buffs = del(buffs)
	end

	local debuffs
	if self.db.profile[frame.group].split then
		debuffs = frame.auraFrame2.debuffs
	else
		debuffs = frame.auraFrame.debuffs
	end
	if debuffs then
		for i = #debuffs, 1, -1 do
			local v = debuffs[i]
			debuffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			if v.border then
				v.border = delFrame(v.border)
			end
			v.unit = nil
			v.id = nil
			v:SetScript('OnEnter', nil)
			v:SetScript('OnLeave', nil)
			delFrame(v)
		end
		if self.db.profile[frame.group].split then
			frame.auraFrame2.debuffs = del(debuffs)
		else
			frame.auraFrame.debuffs = del(debuffs)
		end
	end
end

function PitBull_Aura:OnClearUnitFrame(unit, frame)
	if frame.auraFrame then
		self:ClearAuras(unit, frame)
	
		frame.auraFrame = delFrame(frame.auraFrame)
		if frame.auraFrame2 then
			frame.auraFrame2 = delFrame(frame.auraFrame2)
		end
	end
	if frame.debuffhighlight then
		frame.debuffhighlight = delFrame(frame.debuffhighlight)
	end
end

function PitBull_Aura:OnUnknownLayout(unit, frame, name)
	if name == 'auraFrame' then
		frame.auraFrame:SetHeight(self.db.profile[frame.group].buffsize)
		frame.auraFrame:SetPoint('TOPLEFT', frame, 'BOTTOMLEFT')
		frame.auraFrame:SetPoint('TOPRIGHT', frame, 'BOTTOMRIGHT')
	elseif name == 'auraFrame2' then
		frame.auraFrame2:SetHeight(self.db.profile[frame.group].debuffsize)
		frame.auraFrame2:SetPoint('TOPLEFT', frame.auraFrame, 'BOTTOMLEFT')
		frame.auraFrame2:SetPoint('TOPRIGHT', frame.auraFrame, 'BOTTOMRIGHT')
	elseif name == 'debuffhighlight' then
		frame.debuffhighlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
		frame.debuffhighlight:SetBlendMode("ADD")
		frame.debuffhighlight:SetAlpha(0.5)
		frame.debuffhighlight:SetAllPoints( frame )
		frame.debuffhighlight:Hide()
	end
end
--
function PitBull_Aura:OnUpdateLayout(unit, frame)
	local old = not unitsShown[unit]
	local now = not frame.auraFrame or not frame.auraFrame:IsShown()
	if old == now then
		return
	end
	unitsShown[unit] = not now
	if now then
		self:ClearAuras(unit, frame)
	else
		self:UpdateAuras(unit, frame)
	end
end

local function getSplit(group)
	return PitBull_Aura.db.profile[group].split
end

local function setSplit(group, value)
	PitBull_Aura.db.profile[group].split = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getSnap(group)
	return PitBull_Aura.db.profile[group].snap
end

local function setSnap(group, value)
	PitBull_Aura.db.profile[group].snap = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getHidden(group)
	return PitBull_Aura.db.profile[group].hidden
end

local function setHidden(group, value)
	PitBull_Aura.db.profile[group].hidden = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end


local function getMultiHidden(table)
	local group, type = table.group, table.type
	return PitBull_Aura.db.profile[group]['hidden'..type]
end

local function setMultiHidden(table, value)
	local group, type = table.group, table.type
	PitBull_Aura.db.profile[group]['hidden'..type] = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getHighlight(table)
	local group, type = table.group, table.type
	return PitBull_Aura.db.profile[group].highlight[type]
end

local function setHighlight(table, value)
	local group, type = table.group, table.type
	PitBull_Aura.db.profile[group].highlight[type] = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getTypeHighlight(group)
	return PitBull_Aura.db.profile[group].highlightType
end

local function setTypeHighlight(group, value)
	PitBull_Aura.db.profile[group].highlightType = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getHighlightColor(table)
local type, type2 = table[1], table[2]
	return unpack(type2 and PitBull_Aura.db.profile.colors[type][type2] or PitBull_Aura.db.profile.colors[type])
end

local function setHighlightColor(table, r, g, b, a)
local type, type2 = table[1], table[2]
	if type2 then
		PitBull_Aura.db.profile.colors[type][type2] = {r, g, b, a}
	else
		PitBull_Aura.db.profile.colors[type] = {r, g, b, a}
	end
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(frame)
	end
end

local function getEnableFilter(group)
	return PitBull_Aura.db.profile[group].filter
end

local function setEnableFilter(group, value)
	PitBull_Aura.db.profile[group].filter = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getFrameHighlight(group)
	return PitBull_Aura.db.profile[group].frameHighlight
end

local function setFrameHighlight(group, value)
	PitBull_Aura.db.profile[group].frameHighlight = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getBuffSize(group)
	return PitBull_Aura.db.profile[group].buffsize
end

local function setBuffSize(group, value)
	PitBull_Aura.db.profile[group].buffsize = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getDebuffSize(group)
	return PitBull_Aura.db.profile[group].debuffsize
end

local function setDebuffSize(group, value)
	PitBull_Aura.db.profile[group].debuffsize = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getMaxBuffs(group)
	return PitBull_Aura.db.profile[group].maxbuffs
end

local function setMaxBuffs(group, value)
	PitBull_Aura.db.profile[group].maxbuffs = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getMaxDebuffs(group)
	return PitBull_Aura.db.profile[group].maxdebuffs
end

local function setMaxDebuffs(group, value)
	PitBull_Aura.db.profile[group].maxdebuffs = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getFlipped(group)
	return PitBull_Aura.db.profile[group].flip
end

local function setFlipped(group, value)
	PitBull_Aura.db.profile[group].flip = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getGrowthDirection(group)
	local d = PitBull_Aura.db.profile[group].grow
	if d == 1 then
		return "Up"
	elseif d == -1 then
		return "Down"
	end
end

local function setGrowthDirection(group, value)
	if value == "Up" then
		PitBull_Aura.db.profile[group].grow = 1
	elseif value == "Down" then
		PitBull_Aura.db.profile[group].grow = -1
	end
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

local function getBuffDirection(group)
	return PitBull_Aura.db.profile[group].buffdirection
end

local function setBuffDirection(group, value)
	PitBull_Aura.db.profile[group].buffdirection = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
		PitBull:UpdateLayout(frame)
	end
end

local function getDebuffDirection(group)
	return PitBull_Aura.db.profile[group].debuffdirection
end

local function setDebuffDirection(group, value)
	PitBull_Aura.db.profile[group].debuffdirection = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
		PitBull:UpdateLayout(frame)
	end
end

local function checkSplitOptionsHidden(group)
	return PitBull_Aura.db.profile[group].split
end

local function checkNonSplitOptionsHidden(group)
	return not PitBull_Aura.db.profile[group].split
end

local buffdirectiontable = {
	"Up then Left",
	"Up then Right",
	"Down then Left",
	"Down then Right",
	"Left then Up",
	"Left then Down",
	"Right then Up",
	"Right then Down",
}

PitBull_Aura:RegisterPitBullOptionsMethod(function(group)
	return {
		type = 'group',
		name = 'Auras',
		desc = 'Aura options',
		args = {
			Split = {
				type = 'toggle',
				name = 'Split',
				desc = 'Split buffs and debuffs',
				get = getSplit,
				set = setSplit,
				passValue = group,
			},
			Snap = {
				type = 'toggle',
				name = 'Snap',
				desc = 'Snaps aura sizes to complete rows',
				get = getSnap,
				set = setSnap,
				passValue = group,
			},			
			Hide = {
				type = 'group',
				name = 'Hide',
				desc = 'Hide options',
				args = {
					All = {
						type = 'toggle',
						name = 'All',
						desc = 'Hide Buffs and Debuffs',
						get = getHidden,
						set = setHidden,
						passValue = group,
					},
					Buffs = {
						type = 'toggle',
						name = 'Buffs',
						desc = 'Hide buff bar',
						get = getMultiHidden,
						set = setMultiHidden,
						passValue = {['group'] = group, ['type'] = 'Buffs'}
					},
					Debuffs = {
						type = 'toggle',
						name = 'Debuffs',
						desc = 'Hide debuffs bar',
						get = getMultiHidden,
						set = setMultiHidden,
						passValue = {['group'] = group, ['type'] = 'Debuffs'}
					}
				}
			},
			highlight = {
				type = 'group',
				name = 'Highlight',
				desc = 'Highlight options',
				args = {
					Buffs = {
						type = 'toggle',
						name = 'Buffs',
						desc = 'Highlight buffs',
						get = getHighlight,
						set = setHighlight,
						passValue = {['group'] = group, ['type'] = 'Buffs'}
					},
					Debuffs = {
						type = 'toggle',
						name = 'Debuffs',
						desc = 'Highlight debuffs',
						get = getHighlight,
						set = setHighlight,
						passValue = {['group'] = group, ['type'] = 'Debuffs'}
					},
					DebuffsType = {
						type = 'toggle',
						name = 'Debuff types',
						desc = 'Highlight different debuffs with different colors',
						get = getTypeHighlight,
						set = setTypeHighlight,
						passValue = group,
					},
					Colors = {
						type = 'group',
						name = 'Colors',
						desc = 'Set colors for highlighting',
						args = {
							Buffs = {
								type = 'color',
								name = 'Buffs',
								desc = 'Set color for buffs',
								hasAlpha = true,
								get = getHighlightColor,
								set = setHighlightColor,
								passValue = {'buffs'},
							},
							Debuffs = {
								type = 'group',
								name = 'Debuffs',
								desc = 'Debuff colors',
								args = {
									Curse = {
										type = 'color',
										name = 'Curse',
										desc = 'Set color for curses',
										hasAlpha = true,
										get = getHighlightColor,
										set = setHighlightColor,
										passValue = {'debuffs', 'Curse'},
									},
									Poison = {
										type = 'color',
										name = 'Poison',
										desc = 'Set color for poisons',
										hasAlpha = true,
										get = getHighlightColor,
										set = setHighlightColor,
										passValue = {'debuffs', 'Poison'},
									},
									Magic = {
										type = 'color',
										name = 'Magic',
										desc = 'Set color for magics',
										hasAlpha = true,
										get = getHighlightColor,
										set = setHighlightColor,
										passValue = {'debuffs', 'Magic'},
									},
									Disease = {
										type = 'color',
										name = 'Disease',
										desc = 'Set color for diseases',
										hasAlpha = true,
										get = getHighlightColor,
										set = setHighlightColor,
										passValue = {'debuffs', 'Disease'},
									},
									Other = {
										type = 'color',
										name = 'Other',
										desc = 'Set color for others',
										hasAlpha = true,
										get = getHighlightColor,
										set = setHighlightColor,
										passValue = {'debuffs', 'nil'},
									},
								}
							}
						}
					}
				}
			},
			framehighlight = {
				type = 'text',
				name = 'Frame Highlight',
				desc = 'Debuff Highlighting on the full frame.',
				validate = {"Never", "All Debuffs", "Cureable", "Cureable by me"},
				get = getFrameHighlight,
				set = setFrameHighlight,
				passValue = group,				
			},
			filter = {
				type = 'group',
				name = 'Filtering',
				desc = 'Filter certain buffs and debuffs',
				args =  {
					enable = {
						type = 'toggle',
						name = 'Enable filtering',
						desc = 'Enable filtering',
						get = getEnableFilter,
						set = setEnableFilter,
						passValue = group,
					}
				}
			},
			debuffsize = {
				type = 'range',
				name = 'Debuff Size',
				desc = 'Set size of debuff icons',
				get = getDebuffSize,
				set = setDebuffSize,
				passValue = group,
				min = 4,
				max = 48,
				step = 1,
			},
			buffsize = {
				type = 'range',
				name = 'Buff Size',
				desc = 'Set size of buff icons',
				get = getBuffSize,
				set = setBuffSize,
				passValue = group,
				min = 4,
				max = 48,
				step = 1,
			},
			maxbuffs = {
				type = 'range',
				name = 'Max Buffs',
				desc = 'Set maximum number of buffs to display',
				get = getMaxBuffs,
				set = setMaxBuffs,
				passValue = group,
				min = 0,
				max = 40,
				step = 1,
			},
			maxdebuffs = {
				type = 'range',
				name = 'Max Debuffs',
				desc = 'Set maximum number of debuffs to display',
				get = getMaxDebuffs,
				set = setMaxDebuffs,
				passValue = group,
				min = 0,
				max = 40,
				step = 1,
			},
			direction = {
				type = 'group',
				name = "Direction",
				desc = "Direction Options",
				args = {
					-- options when not in split mode
					flip = {
						type = 'toggle',
						name = 'Flip Buffs/Debuffs',
						desc = 'Flip the positions of buffs and debuffs',
						get = getFlipped,
						set = setFlipped,
						passValue = group,
						hidden = checkSplitOptionsHidden,
					},
					grow = {
						type = 'text',
						name = "Set Growth Direction",
						desc = "Set the growth direction for extra rows of buffs and debuffs",
						get = getGrowthDirection,
						set = setGrowthDirection,
						passValue = group,
						validate = {"Up","Down"},
						hidden = checkSplitOptionsHidden,
					},
					-- options when in split mode
					buffDirection = {
						type = 'text',
						name = "Set Buff Direction",
						desc = "Set the direction in which buffs grow",
						get = getBuffDirection,
						set = setBuffDirection,
						passValue = group,
						validate = buffdirectiontable,
						hidden = checkNonSplitOptionsHidden,
					},
					debuffDirection = {
						type = 'text',
						name = "Set Debuff Direction",
						desc = "Set the direction in which debuffs grow",
						get = getDebuffDirection,
						set = setDebuffDirection,
						passValue = group,
						validate = buffdirectiontable,
						hidden = checkNonSplitOptionsHidden,
					},
				},
			},
		}
	}
end)
