local VERSION = tonumber(("$Revision: 32429 $"):match("%d+"))

local PitBull = PitBull
local PitBull_CastBar = PitBull:NewModule("CastBar", "AceEvent-2.0")
local self = PitBull_CastBar
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-12 05:54:17 -0400 (Thu, 12 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = setmetatable({}, {__index = function(self, key) self[key] = key; return key end })

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_CastBar:RegisterPitBullChildFrames('castBar', 'castBarText', 'castBarTimeText', 'castBarIcon')

local castData = {}

function PitBull_CastBar:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("CastBar")
	PitBull:RegisterDefaults("CastBar", "profile", {
		groups = {
			['**'] = {},
			["targettarget"] = {
				ignore = true,
			},
			["targettargettarget"] = {
				ignore = true,
			},
			["focustarget"] = {
				ignore = true,
			},			
			["focustarget"] = {
				ignore = true,
			},
			["partytarget"] = {
				ignore = true,
			},			
			["partypettarget"] = {
				ignore = true,
			},			
			["raidtarget"] = {
				ignore = true,
			},
		},
		spelltextcolor = {1, 1, 1},
		timetextcolor = {1, 1, 1},
		castingcolor = {1.0, 0.7, 0.0},
		channelingcolor = {0, 1.0, 0},
		completecolor = {0, 1.0, 0},
		failcolor = {1.0, 0, 0},
		backgroundcolor = { .5, .5, .5, .8 },
	})
end

function PitBull_CastBar:OnEnable(first)
	self.castData = castData -- for Roartindon so he can read the data.
	self:ScheduleRepeatingEvent("PitBull_CastBar-Update", self.UpdateAllCastBars, 0, self)

	self:RegisterEvent("UNIT_SPELLCAST_SENT")	
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_INTERRUPTED", "UNIT_SPELLCAST_INTERRUPTED")
end

function PitBull_CastBar:UpdateAllCastBars()
	local frame
	for unit, data in pairs(castData) do
		frame = PitBull:GetUnitFrame(unit)
		if frame then
			local currentTime = GetTime()
			if data.casting then
				local showTime = math.min(currentTime, data.endTime)
				frame.castBar:SetValue(showTime)
				local delay = data.delay
				if delay and delay ~= 0 then
					frame.castBarTimeText:SetText(("|cffff0000+%.1f|cffffffff %.1f"):format(data.delay, data.endTime - showTime))
				else
					frame.castBarTimeText:SetText(("%.1f"):format(data.endTime - showTime))
				end
				if currentTime > data.endTime and not UnitIsUnit("player", unit) then
					castData[unit].casting = nil
					castData[unit].fadeOut = 1
					castData[unit].stopTime = currentTime
				end
			elseif data.channeling then
				local showTime = currentTime
				if currentTime > data.endTime then
					showTime = data.endTime
					castData[unit].channeling = nil
					castData[unit].fadeOut = 1
					castData[unit].stopTime = currentTime
				end
				local remainingTime = data.endTime - showTime
				frame.castBar:SetValue(data.startTime + remainingTime)
				frame.castBarTimeText:SetText(("%.1f"):format(remainingTime))

				if data.delay and data.delay ~= 0 then
					frame.castBarTimeText:SetText(("|cffFF0000-%.1f|cffffffff %.1f"):format(data.delay, remainingTime))
				else
					frame.castBarTimeText:SetText(("%.1f"):format(remainingTime))
				end
			elseif data.fadeOut then
				local alpha
				if data.stopTime then
					alpha = data.stopTime - currentTime + 1
				else
					alpha = 0
				end
				if alpha >= 1 then
					alpha = 1
				end
				if alpha <= 0 then
					frame.castBar:Hide()
					frame.castBarText:SetText("")
					frame.castBarTimeText:SetText("")
					frame.castBarIcon:Hide()
					frame.castBar:SetValue(0)
					
					castData[unit] = del(castData[unit])
				else
					frame.castBar:SetAlpha(alpha)
					frame.castBarText:SetAlpha(alpha)
					frame.castBarTimeText:SetAlpha(alpha)
					frame.castBarIcon:SetAlpha(alpha)
				end
			else
				castData[unit] = del(castData[unit])
			end
		else
			castData[unit] = del(castData[unit])
		end
	end
end

function PitBull_CastBar:UNIT_SPELLCAST_SENT(unit, spell, rank, target)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or self.db.profile.groups[frame.group].ignore then
		return
	end

	if not castData[unit] then
		castData[unit] = new()
	end

	if target == "" then
		target = nil
	end
	castData[unit].target = target
end

function PitBull_CastBar:UNIT_SPELLCAST_START(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or self.db.profile.groups[frame.group].ignore then
		return
	end

	if not castData[unit] then
		castData[unit] = new()
	end
	local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)

	castData[unit].spell = spell
	castData[unit].rank = rank
	castData[unit].displayName = displayName
	castData[unit].icon = icon
	castData[unit].startTime = startTime * 0.001 
	castData[unit].endTime = endTime * 0.001
	castData[unit].delay = 0
	castData[unit].casting = 1
	castData[unit].channeling = nil
	castData[unit].fadeOut = nil

	frame.castBar:SetStatusBarColor(unpack(self.db.profile.castingcolor))
	frame.castBar:SetAlpha(1.0)
	frame.castBar:SetMinMaxValues(castData[unit].startTime, castData[unit].endTime)
	-- calculate the correct value since this function can be called for wacky units as well.
	frame.castBar:SetValue(GetTime())
	frame.castBar:Show()

	if type(castData[unit].target)=="string" and UnitName(unit) ~= castData[unit].target then
		frame.castBarText:SetText(displayName .. " (" .. castData[unit].target ..")" )
	else
		frame.castBarText:SetText(displayName)
	end
	frame.castBarText:SetAlpha(1.0)
	frame.castBarText:Show()
	frame.castBarText:SetTextColor(unpack(self.db.profile.spelltextcolor))
	
	frame.castBarTimeText:SetAlpha(1.0)
	frame.castBarTimeText:Show()
	frame.castBarTimeText:SetTextColor(unpack(self.db.profile.timetextcolor))

	frame.castBarIcon:SetAlpha(1.0)
	frame.castBarIcon:SetTexture(icon)
	frame.castBarIcon:Show()
end

function PitBull_CastBar:UNIT_SPELLCAST_CHANNEL_START(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or self.db.profile.groups[frame.group].ignore then
		return
	end

	if not castData[unit] then
		castData[unit] = newHash()
	end
	local spell, rank, displayName, icon, startTime, endTime = UnitChannelInfo(unit)

	castData[unit].spell = spell
	castData[unit].rank = rank
	castData[unit].displayName = spell
	castData[unit].icon = icon
	castData[unit].startTime = startTime * 0.001
	castData[unit].endTime = endTime * 0.001
	castData[unit].delay = 0
	castData[unit].casting = nil
	castData[unit].channeling = 1
	castData[unit].fadeOut = nil

	frame.castBar:SetStatusBarColor(unpack(self.db.profile.channelingcolor))
	frame.castBar:SetAlpha(1.0)
	frame.castBar:SetMinMaxValues(castData[unit].startTime, castData[unit].endTime)
	-- calculate the correct value since this function can be called for wacky units as well.	
	frame.castBar:SetValue(castData[unit].startTime + castData[unit].endTime - GetTime())
	frame.castBar:Show()


	frame.castBarText:SetText(spell)
	frame.castBarText:SetAlpha(1.0)
	frame.castBarText:Show()
	frame.castBarText:SetTextColor(unpack(self.db.profile.spelltextcolor))

	frame.castBarTimeText:SetAlpha(1.0)
	frame.castBarTimeText:Show()
	frame.castBarTimeText:SetTextColor(unpack(self.db.profile.timetextcolor))

	frame.castBarIcon:SetAlpha(1.0)
	frame.castBarIcon:SetTexture(icon)
	frame.castBarIcon:Show()
end

function PitBull_CastBar:UNIT_SPELLCAST_STOP(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or	not castData[unit] or not castData[unit].casting then
		return
	end

	castData[unit].casting = nil
	castData[unit].fadeOut = 1
	castData[unit].stopTime = GetTime()

	frame.castBar:SetStatusBarColor(unpack(self.db.profile.completecolor))
	frame.castBar:SetMinMaxValues(0.0, 1.0)
	frame.castBarTimeText:SetText("")

end

function PitBull_CastBar:UNIT_SPELLCAST_FAILED( unit )
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or not castData[unit] or (not castData[unit].casting and not castData[unit].channeling) then 
		return
	end

	castData[unit].casting = nil
	castData[unit].channeling = nil
	castData[unit].fadeOut = 1
	castData[unit].stopTime = GetTime()

	frame.castBar:SetStatusBarColor(unpack(self.db.profile.failcolor))
	frame.castBar:SetAlpha(1.0)
	frame.castBar:SetMinMaxValues(0.0, 1.0)
	frame.castBar:SetValue(1.0)
	frame.castBar:Show()
	
	frame.castBarText:SetText(FAILED)
	frame.castBarText:SetAlpha(1.0)
	frame.castBarText:Show()

end

function PitBull_CastBar:UNIT_SPELLCAST_INTERRUPTED(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or	not castData[unit] then
		return
	end

	castData[unit].casting = nil
	castData[unit].channeling = nil
	castData[unit].fadeOut = 1
	castData[unit].stopTime = GetTime()

	frame.castBar:SetStatusBarColor(unpack(self.db.profile.failcolor))
	frame.castBar:SetAlpha(1.0)
	frame.castBar:SetMinMaxValues(0.0, 1.0)
	frame.castBar:SetValue(1.0)
	frame.castBar:Show()
	
	frame.castBarText:SetText(INTERRUPTED)
	frame.castBarText:SetAlpha(1.0)
	frame.castBarText:Show()

end

function PitBull_CastBar:UNIT_SPELLCAST_DELAYED(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or not castData[unit] or not castData[unit].casting then
		return
	end

	local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
	
	if not spell or not startTime or not endTime then
		return
	end

	local oldStart = castData[unit].startTime

	castData[unit].startTime = startTime * 0.001 
	castData[unit].endTime = endTime * 0.001 

	castData[unit].delay = (castData[unit].delay or 0) + (castData[unit].startTime - oldStart)
	
	frame.castBar:SetMinMaxValues(castData[unit].startTime, castData[unit].endTime)
end

function PitBull_CastBar:UNIT_SPELLCAST_CHANNEL_UPDATE(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or not castData[unit] then
		return
	end

	local spell, rank, displayName, icon, startTime, endTime = UnitChannelInfo(unit)

	if not spell then
		castData[unit] = del(castData[unit])
		frame.castBarText:SetText("")
		frame.castBarTimeText:SetText("")
		frame.castBarIcon:Hide()
		frame.castBar:Hide()
		return
	end
	
	local oldStart = castData[unit].startTime
	castData[unit].startTime = startTime * 0.001
	castData[unit].endTime = endTime * 0.001
	castData[unit].delay = (castData[unit].delay or 0) + (oldStart - castData[unit].startTime)

	frame.castBar:SetMinMaxValues(castData[unit].startTime, castData[unit].endTime)
end

function PitBull_CastBar:UNIT_SPELLCAST_CHANNEL_STOP(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if not frame or not castData[unit] then 
		return
	end

	castData[unit].channeling = nil
	castData[unit].casting = nil
	castData[unit].fadeOut = 1
	castData[unit].stopTime = GetTime()
	
	frame.castBarTimeText:SetText("")
end

function PitBull_CastBar:OnUpdateFrame(unit, frame)
	if not frame.castBar or self.db.profile.groups[frame.group].ignore then
		return
	end

	if PitBull.IsWackyUnit[unit] then		
		-- We have a non-event driven wacky unit with a castBar that's not ignored 
		-- We manually call the start and stop functions for those units
		local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
		if displayName then
			self:UNIT_SPELLCAST_START(unit)
			return
		end
		spell, rank, displayName, icon, startTime, endTime = UnitChannelInfo(unit)
		if displayName then
			self:UNIT_SPELLCAST_CHANNEL_START(unit)
			return
		end
		if castData[unit] then
			if castData[unit].channeling then
				self:UNIT_SPELLCAST_CHANNEL_STOP(unit)
			elseif castData[unit].casting then
				self:UNIT_SPELLCAST_STOP(unit)
			end
		end
	elseif unit == "target" or unit == "focus" then
		-- This occurs when the player has selected a new target or focus.
		local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
		if displayName then
			self:UNIT_SPELLCAST_START(unit)
			return
		end
		spell, rank, displayName, icon, startTime, endTime = UnitChannelInfo(unit)
		if displayName then
			self:UNIT_SPELLCAST_CHANNEL_START(unit)
			return
		end
		if castData[unit] then
			castData[unit] = del(castData[unit])
			frame.castBar:Hide()
			frame.castBarText:SetText("")
			frame.castBarTimeText:SetText("")
			frame.castBarIcon:Hide()
			frame.castBar:SetValue(0)
		end
	end
end

function PitBull_CastBar:OnPopulateUnitFrame(unit, frame)
	if frame.castBar or self.db.profile.groups[frame.group].ignore then
		return 
	end
	local castBar = newFrame("StatusBar", frame)
	frame.castBar = castBar
	castBar:SetStatusBarTexture(PitBull:GetStatusBarTexture())
	castBar:SetMinMaxValues(0, 1)
	castBar:SetValue(0)
	castBar:Hide()
	
	local castBarBG = newFrame("Texture", frame, "BACKGROUND")
	frame.castBar.bg = castBarBG
	castBarBG:SetTexture(PitBull:GetStatusBarTexture())
	castBarBG:SetVertexColor(unpack(self.db.profile.backgroundcolor))
	castBarBG:ClearAllPoints()
	castBarBG:SetAllPoints(castBar)
	castBarBG:Hide()
	
	local font, fontsize = PitBull:GetFont()
	
	local castBarText = newFrame("FontString", castBar, "OVERLAY")
	frame.castBarText = castBarText
	castBarText:SetFont( font, fontsize )
	castBarText:SetShadowColor( 0, 0, 0, 1)
	castBarText:SetShadowOffset( 0.8, -0.8 )
	castBarText:SetNonSpaceWrap(false)
	castBarText:SetJustifyH( "LEFT" )
	castBarText:ClearAllPoints()
	castBarText:SetAllPoints(castBar)
	castBarText:Hide()

	local castBarTimeText = newFrame("FontString", castBar, "OVERLAY")
	frame.castBarTimeText = castBarTimeText
	castBarTimeText:SetFont( font, fontsize )
	castBarTimeText:SetShadowColor( 0, 0, 0, 1)
	castBarTimeText:SetShadowOffset( 0.8, -0.8 )
	castBarTimeText:SetNonSpaceWrap(false)
	castBarTimeText:SetJustifyH("RIGHT")
	castBarTimeText:ClearAllPoints()
	castBarTimeText:SetAllPoints(castBar)
	castBarTimeText:Hide()

	local castBarIcon = newFrame("Texture", castBar, "BACKGROUND")
	frame.castBarIcon = castBarIcon
	castBarIcon:SetPoint( "RIGHT", castBar, "LEFT", 0, 0)
end

function PitBull_CastBar:OnUpdateFont(font, fontsize)
	for _,frame in PitBull:IterateUnitFrames() do
		if frame.castBar then
			frame.castBarText:SetFont( font, fontsize)
			frame.castBarTimeText:SetFont( font, fontsize )
		end
	end
end

function PitBull_CastBar:OnUpdateStatusBarTexture(texture)
	for _,frame in PitBull:IterateUnitFrames() do
		if frame.castBar then
			frame.castBar:SetStatusBarTexture(texture)
			frame.castBar.bg:SetTexture(texture)
		end
	end
end

function PitBull_CastBar:OnClearUnitFrame(unit, frame)
	if frame.castBar then
		frame.castBar.bg = delFrame(frame.castBar.bg)
		frame.castBarText = delFrame(frame.castBarText)
		frame.castBarTimeText = delFrame(frame.castBarTimeText)
		frame.castBarIcon = delFrame(frame.castBarIcon)
		frame.castBar = delFrame(frame.castBar)
	end
end

function PitBull_CastBar:OnUnknownLayout(unit, frame, name)
	if frame.castBar then
		if name == 'castBar' then
			frame.castBar:SetHeight(16)
			frame.castBar:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 21, 0 )
			frame.castBar:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -5, 0)
			frame.castBar.bg:ClearAllPoints()
			frame.castBar.bg:SetAllPoints(frame.castBar)
			frame.castBar:Hide()
		elseif name == 'castBarIcon' then
			frame.castBarIcon:SetHeight(16)
			frame.castBarIcon:SetWidth(16)
			frame.castBarIcon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
			frame.castBarIcon:SetPoint("RIGHT", frame.castBar, "LEFT")
		elseif name == 'castBarText' then
			frame.castBarText:ClearAllPoints()
			frame.castBarText:SetAllPoints(frame.castBar)
		elseif name == 'castBarTimeText' then
			frame.castBarTimeText:ClearAllPoints()
			frame.castBarTimeText:SetAllPoints(frame.castBar)
			frame.castBarTimeText:SetJustifyH("RIGHT")
		end
	end
end
-- Global Options
local function getDBColor(field)
	return unpack(PitBull_CastBar.db.profile[field])
end
local function setDBColor(field,r,g,b)
	PitBull_CastBar.db.profile[field] = {r,g,b}
end
local function setDBColorWithAlpha(field,r,g,b,a)
	PitBull_CastBar.db.profile[field] = {r,g,b,a}
	if field == 'backgroundcolor' then
		for _,frame in PitBull:IterateUnitFrames() do
			if frame.castBar then
				frame.castBar.bg:SetVertexColor(r, g, b, a)
			end
		end
	end
end
PitBull.options.args.global.args.castbar = {
	type = 'group',
	name = L["Cast Bar"],
	desc = L["Cast Bar"],
	args = {
		spelltextcolor = {
			type = 'color',
			name = L["Spell Text Color"],
			desc = L["Spell Text Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "spelltextcolor",
		},
		timetextcolor = {
			type = 'color',
			name = L["Time Text Color"],
			desc = L["Time Text Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "timetextcolor",
		},
		castingcolor = {
			type = 'color',
			name = L["Casting Color"],
			desc = L["Casting Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "castingcolor",
		},
		channelingcolor = {
			type = 'color',
			name = L["Channeling Color"],
			desc = L["Channeling Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "channelingcolor",
		},
		completecolor = {
			type = 'color',
			name = L["Complete Color"],
			desc = L["Complete Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "completecolor",
		},
		failcolor = {
			type = 'color',
			name = L["Fail Color"],
			desc = L["Fail Color"],
			get = getDBColor,
			set = setDBColor,
			passValue = "failcolor",
		},
		backgroundcolor = {
			type = 'color',
			name = L["Background Color"],
			desc = L["Background Color"],
			get = getDBColor,
			set = setDBColorWithAlpha,
			passValue = "backgroundcolor",
			hasAlpha = true,
		},
	},
}
-- Unit/Group Specific Options
local function getIgnored(group)
	return PitBull_CastBar.db.profile.groups[group].ignore
end
local function setIgnored(group, value)
	PitBull_CastBar.db.profile.groups[group].ignore = value
	if value then
		for unit,frame in PitBull:IterateUnitFramesByGroup(group) do
			if frame.castBar then
				PitBull_CastBar:OnClearUnitFrame(unit, frame)
				castData[unit] = nil
				PitBull:UpdateLayout(frame)
			end
		end
	else
		for unit,frame in PitBull:IterateUnitFramesByGroup(group) do
			if not frame.castBar then
				PitBull_CastBar:OnPopulateUnitFrame(unit, frame)
				PitBull:UpdateLayout(frame)
			end
		end
	end
end

PitBull_CastBar:RegisterPitBullOptionsMethod(function(group)
	return {
		name = L["Cast Bar"],
		desc = L["Cast Bar"],
		type = 'group',
		args = {
			ignore = {
				type = 'toggle',
				name = L["Disable Cast Bar"],
				desc = L["Disable Cast Bar"],
				get = getIgnored,
				set = setIgnored,
				passValue = group,
			},
		}
	}
end)
